import _ from 'lodash'

import { BitbucketServerApi } from './bitbucket-server.api'
import { GitEnricherInputs } from '../../../image-enricher-git-info/types'

export class BitbucketServerService {
    private bitBucketServerApi

    constructor(inputs: GitEnricherInputs) {
        this.bitBucketServerApi = new BitbucketServerApi(inputs)
    }

    async getPullRequestsWithCommits(repo, branch) {
        try {
            const repoId = await this.getRepoId(repo)

            const prs = await this.bitBucketServerApi.getPullRequests(repoId, branch)
            return Promise.all(prs.map(async (pr) => {
                const info = await this.bitBucketServerApi.getCommitsInfo(repoId, pr)
                const result = {
                    ...info,
                    prDate: pr.updatedDate || pr.createdDate,
                    number: pr.id,
                    url: `${pr.links.self[0].href}`,
                    title: pr.title,
                }

                return result
            }))
        } catch (error) {
            throw new Error(`failed to get pull requests: ${error.message}`)
        }
    }

    async getBranch(repo, branch) {
        try {
            const repoId = await this.getRepoId(repo)

            const bitBucketBranch = await this.bitBucketServerApi.getBranch(repoId, branch)
            if (!bitBucketBranch) {
                return null
            }
            return {
                name: branch,
                commit: bitBucketBranch.id,
                commitMsg: bitBucketBranch.message,
                commitURL: bitBucketBranch.commitURL,
                committerUsername: _.get(bitBucketBranch, 'author.emailAddress')
            }
        } catch (error) {
            if (_.get(error, 'statusCode') === 404) {
                // branch could be delete, but PR still can be found
                return
            }
            throw new Error(`failed to get branch: ${error.message}`)
        }
    }

    async getRepoId(repoInput) {
        const repo = await this.bitBucketServerApi.getRepo(repoInput)
        return `${repo.project.key}/${repo.slug}`
    }
}
