import _ from 'lodash'

import { GithubApi } from './github.api'
import { GitEnricherInputs } from '../../../image-enricher-git-info/types'

export class GithubService {
    private githubApi

    constructor(inputs: GitEnricherInputs) {
        this.githubApi = new GithubApi(inputs)
    }

    async getCommitsInfo(repo, number) {
        return this.githubApi.getCommitsInfo(repo, number)
    }

    async getPullRequestsWithCommits(repo, branch) {
        try {
            const searchResult = await this.githubApi.searchForPullRequests(repo, branch)

            return Promise.all(searchResult.items.map(async (foundItem) => {
                const commitsInfo = await this.githubApi.getCommitsInfo(repo, foundItem.number)
                const result = {
                    ...commitsInfo,
                    prDate: foundItem.closed_at || foundItem.updated_at,
                    number: foundItem.number,
                    url: foundItem.pull_request.html_url,
                    title: foundItem.title,
                }

                return result
            }))
        } catch (error) {
            throw new Error(`failed to get pull requests: ${error.message}`)
        }
    }

    async getBranch(repo, branch, revision) {
        try {
            const githubBranch = await this.githubApi.getBranch(repo, branch)

            let commitByRevision
            if (revision && githubBranch.commit.sha !== revision) {
                console.log(`Getting commit by revision ${revision}`)
                commitByRevision = await this.githubApi.getCommit(repo, branch, revision)
                  .catch(error => console.error(`Can't get commit by revision ${revision}. Using branch. ${error.message}`))
            }

            const commit = commitByRevision || githubBranch.commit

            return {
                name: githubBranch.name,
                commit: commit.sha,
                commitMsg: commit.commit.message,
                commitURL: commit.html_url,
                committerUsername: commit.author.login
            }
        } catch (error) {
            if (_.get(error, 'status') === 404) {
                // branch could be delete, but PR still can be found
                return
            }
            throw new Error(`failed to get branch: ${error.message}`)
        }
    }
}
